import {
  Component,
  OnInit,
  ChangeDetectorRef,
  OnDestroy,
  Output,
  EventEmitter
} from '@angular/core';
import {
  SelectedDept,
  UserInfoSS,
  AuthResponse
} from '@ucap-webmessenger/protocol-query';
import { NGXLogger } from 'ngx-logger';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from '@ucap-webmessenger/ui';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { Store, select } from '@ngrx/store';

import * as AppStore from '@app/store';
import * as SettingsStore from '@app/store/messenger/settings';
import * as SyncStore from '@app/store/messenger/sync';
import * as ChatStore from '@app/store/messenger/chat';

import { Observable, Subscription } from 'rxjs';
import { PresenceType } from '@ucap-webmessenger/core';
import { StatusBulkInfo } from '@ucap-webmessenger/protocol-status';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { KEY_VER_INFO, KEY_AUTH_INFO, MainMenu } from '@app/types';

import { LoginResponse } from '@ucap-webmessenger/protocol-authentication';
import { tap } from 'rxjs/operators';
import {
  SelectGroupDialogComponent,
  SelectGroupDialogData,
  SelectGroupDialogResult
} from '../../dialogs/group/select-group.dialog.component';
import { GroupDetailData } from '@ucap-webmessenger/protocol-sync';

@Component({
  selector: 'app-layout-messenger-organization',
  templateUrl: './organization.component.html',
  styleUrls: ['./organization.component.scss']
})
export class OrganizationComponent implements OnInit, OnDestroy {
  @Output()
  openProfile = new EventEmitter<number>();
  @Output()
  createConference = new EventEmitter<number[]>();

  loginRes: LoginResponse;
  loginResSubscription: Subscription;
  sessionVerinfo: VersionInfo2Response;
  authInfo: AuthResponse;

  isSearch: boolean;
  isSearchSubscription: Subscription;
  selectedDepartmentProcessing$: Observable<boolean>;
  selectedDepartment$: Observable<SelectedDept>;

  departmentUserInfoListSubscription: Subscription;
  searchDepartmentUserInfoListSubscription: Subscription;

  departmentUserInfoList: UserInfoSS[] = [];

  selectedUserList: UserInfoSS[] = []; // selected user in departmentUserList detail

  profileImageRoot: string;
  presence$: Observable<StatusBulkInfo[]>;
  PresenceType = PresenceType;

  constructor(
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private changeDetectorRef: ChangeDetectorRef,
    private logger: NGXLogger
  ) {
    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
    this.authInfo = this.sessionStorageService.get<AuthResponse>(KEY_AUTH_INFO);
  }

  ngOnInit() {
    this.loginResSubscription = this.store
      .pipe(
        select(AppStore.AccountSelector.AuthenticationSelector.loginRes),
        tap(loginRes => {
          this.loginRes = loginRes;
        })
      )
      .subscribe();

    this.isSearchSubscription = this.store
      .pipe(select(AppStore.MessengerSelector.QuerySelector.isSearch))
      .subscribe(isSearch => {
        this.isSearch = isSearch;
      });

    this.selectedDepartmentProcessing$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.selectedDepartmentProcessing
      )
    );

    this.selectedDepartment$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.selectedDepartment)
    );

    this.departmentUserInfoListSubscription = this.store
      .pipe(
        select(AppStore.MessengerSelector.QuerySelector.departmentUserInfoList)
      )
      .subscribe(list => {
        if (!this.isSearch) {
          this.departmentUserInfoList = list;
        }
      });

    this.searchDepartmentUserInfoListSubscription = this.store
      .pipe(
        select(
          AppStore.MessengerSelector.QuerySelector.searchDepartmentUserInfoList
        )
      )
      .subscribe(list => {
        if (!!this.isSearch) {
          this.departmentUserInfoList = list;
        }
      });

    this.presence$ = this.store.pipe(
      select(AppStore.MessengerSelector.StatusSelector.selectAllStatusBulkInfo)
    );

    this.profileImageRoot = this.sessionVerinfo.profileRoot;
  }

  ngOnDestroy(): void {
    if (!!this.loginResSubscription) {
      this.loginResSubscription.unsubscribe();
    }
    if (!!this.isSearchSubscription) {
      this.isSearchSubscription.unsubscribe();
    }
    if (!!this.departmentUserInfoListSubscription) {
      this.departmentUserInfoListSubscription.unsubscribe();
    }
    if (!!this.searchDepartmentUserInfoListSubscription) {
      this.searchDepartmentUserInfoListSubscription.unsubscribe();
    }
  }

  /** Selected User Handling */
  onToggleAllUser(params: { isChecked: boolean; userInfos: UserInfoSS[] }) {
    params.userInfos.forEach(userInfo => {
      if (params.isChecked) {
        if (
          this.selectedUserList.filter(user => user.seq === userInfo.seq)
            .length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, userInfo];
        }
      } else {
        this.selectedUserList = this.selectedUserList.filter(
          user => user.seq !== userInfo.seq
        );
      }
    });
  }
  onToggleUser(userInfo: UserInfoSS) {
    if (userInfo.seq === this.loginRes.userSeq) {
      return;
    }

    if (
      this.selectedUserList.filter(user => user.seq === userInfo.seq).length ===
      0
    ) {
      this.selectedUserList = [...this.selectedUserList, userInfo];
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        item => item.seq !== userInfo.seq
      );
    }
    this.changeDetectorRef.detectChanges();
  }

  /** Handling chipset for selectedUserList */
  /** 선택된 사용자 취소 */
  onClickDeleteUser(userInfo: UserInfoSS) {
    this.selectedUserList = this.selectedUserList.filter(
      item => item.seq !== userInfo.seq
    );
    this.changeDetectorRef.detectChanges();
  }

  /** Handling Button */
  async onClickAddGroup() {
    this.logger.debug('onClickAddGroup', this.selectedUserList);

    const result = await this.dialogService.open<
      SelectGroupDialogComponent,
      SelectGroupDialogData,
      SelectGroupDialogResult
    >(SelectGroupDialogComponent, {
      width: '600px',
      data: {
        title: this.translateService.instant('group.selectTargetGroup')
      }
    });

    if (!!result && !!result.choice && result.choice) {
      if (!!result.group) {
        const oldGroup: GroupDetailData = result.group;
        const trgtUserSeq: number[] = [];
        result.group.userSeqs.map(seq => trgtUserSeq.push(seq));
        this.selectedUserList
          .filter(v => result.group.userSeqs.indexOf(v.seq) < 0)
          .forEach(user => {
            trgtUserSeq.push(user.seq);
          });

        this.store.dispatch(
          SyncStore.updateGroupMember({
            oldGroup,
            trgtUserSeq
          })
        );
      }
    }
  }
  onClickChatOpen() {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      // Open Room.
      const seq: number[] = [];
      this.selectedUserList.map(user => seq.push(user.seq));
      this.store.dispatch(ChatStore.openRoom({ userSeqList: seq }));

      // GNB Change to Chat
      this.store.dispatch(
        SettingsStore.selectedGnbMenuIndex({
          menuIndex: MainMenu.Chat
        })
      );
    }
  }
  onClickConference() {
    const targetUserSeqs = this.selectedUserList.map(userInfo => userInfo.seq);
    this.createConference.emit(targetUserSeqs);
  }

  onClickOpenProfile(userSeq: number) {
    this.openProfile.emit(userSeq);
  }
}
